//.............................................................................
//                                                                            :
//   Program Name: Dialog.Prg        Copyright: Prodigy Consulting            :                           
//   Date Created: 03/14/92           Language: Clipper 5.0                   :                      
//   Time Created: 11:33:08             Author: Mark Vivanco                  :            
//   Version     : 1.3                                                        :
//.............................................................................
//: Modifications                                                             :
//:      10/04/92    Added the following options:                             :
//:                  lNoEsc - Wether to allow the escape key to exit the      :
//:                           dialog box.                                     :
//:                  lCenterMessages - Wether to allow the messages to be     :
//:                           centered in the dialog box.                     :
//:                  nBoxType - The box border type to use for the dialog box :
//:                  I also created a command interface for the dialog box    :
//:      08/18/92    Added the ability to have the colors passed as either    :
//:                  a color number or as a color string.  (In Artful version):
//:      08/11/92    Added the ability to specify default option and hotkeys  :
//:                  These modifications will make it easier to see which     :
//:                  option is hilited.  Also these modifications will allow  :
//:                  hotkeys from the middle of the button and not just the   :
//:                  first letter.                                            :
//.............................................................................
#include "mark.ch"
#include "inkey.ch"

STATIC nFirstDispLine                           // First Displayed Line number
STATIC nLastDispLine                            // Last Displayed Line number
STATIC cSayColor                                // The text/box Say color
STATIC nTotNumLines                             // Total Number of lines in aMessage

#define NUMSPACES       3                       // The number of spaces between buttons
#define LEFTCHAR        Chr(16)                 // The left character for the buttons
#define RIGHTCHAR       Chr(17)                 // The right character for the buttons
#define D_DEFAULTOPTION 1                       // The default option number
#define D_SHADOWS       .T.                     // The default is to have shadows
#define D_BOXCOLOR      "W+/B"                  // The default Box color
#define D_HDRCOLOR      "B/W*"                  // The default Header color
#define D_BUTTONCOLOR   "N/BG,N/BG*"            // The default button color
#define D_HOTKEYCOLOR   "R"                     // The default foreground color for hotkey
#define D_ALLOWESC      .F.                     // Do not allow esc key to exit the dialog box
#define K_SPACE         32                      // Key code for Space bar

// This is a test of the dialogbox and mv_pop_msg functions
#ifdef DIALOG_TEST
#include "dialog.ch"
FUNCTION Test
   LOCAL nResult:=0    
   LOCAL HoldScreen:=GetScreen(0, 0, 24, 79)
   LOCAL HoldCrsPos:=SaveCursorPos()

   DIALOG MESSAGE "", "Here is a demo with all the options used." , ;
                  "I am using the message, options, header,"      , ;
                  "footer, boxcolor, headercolor, buttoncolor,"   , ;
                  "shadows, top, left, bottom, hotkeycolor,"      , ;
                  "default, centermessages, noesc, boxtype and"   , ;
                  "the to parameters.  Thats all folks!"            ;
          OPTIONS "Option ~1", "Option ~2" ;
          HEADER  " This is a sample header "       ;
          FOOTER  " Sample footer " ;
          BOXCOLOR "W+/B"           ;
          HEADERCOLOR "RG*/W"       ;   
          BUTTONCOLOR "N/BG,N/BG*"  ;
          HOTKEYCOLOR "R"           ;
          SHADOWS                   ;
          TOP 5                     ;
          BOTTOM 12                 ;
          LEFT 15                   ;
          DEFAULT 2                 ;
          CENTERMESSAGES            ;
          NOESC                     ;
          BOXTYPE 7                 ;
          TO nResult              

          DIALOG MESSAGE "You chose option " + Ltrim(Str(nResult)) ;
          SHADOWS ;
          HEADER " Result "

   PutScreen(HoldScreen)
   RestCursorPos(HoldCrsPos)
RETURN ( NIL )
#endif


*  *
FUNCTION DialogBox(aMessage, acOptions, cHeader, cFooter, cBoxColor, cHdrColor, ;
         cButtonColor, lShadow, nBoxTR, nBoxTC, nBoxBR, cHotKeyColor, nDefaultOption, ;
         lAllowEsc, lCenterMessages, nBoxType)
*  *
// aMessage       = An array of message lines
// acOptions      = An array of Options to be selected, if not passed then an
//                  "Okay" button will be added automatically.
// cHeader        = The header line of the Dialog Box
// cFooter        = The footer line of the Dialog Box
// cBoxColor      = The color number as associated with SysColor()
// cHdrColor      = The color number as associated with SysColor()
// cButtonColor   = The color number as associated with SysColor()
// lShadow        = Do you want Shadows to appear around the buttons and window?
// nBoxTR         = The top row position of the dialog box.
// nBoxTC         = The top col position of the dialog box.           
// nBoxBR         = The bottom row of the dialog box.
// cHotKeyColor   = The hot key foreground color.  Only a R, B, BG, etc.. not BG/W!
// nDefaultOption = The default first option to hilite.
// lAllowEsc      = Should we allow the Esc key to abort the Dialog Box?
// lCenterMessages= Should we center all the messages?
// nBoxType       = The box border type to use for the dialog box.

LOCAL nBoxBC                                     // Box co-ordinates
LOCAL nNumLines                                  // Number of lines in aMessage
LOCAL nOptions                                   // Number of options
LOCAL anOptCols:={}                              // Column numbers for Options
LOCAL nWidestLine:=0                             // Widest Text line to print
LOCAL nLenOfOptions:=0                           // Length of Option Line
LOCAL nMaxRow:=MAXROW(), nMaxCol:=MaxCol()       // Maximum Screen Rows & Cols
LOCAL nBegOpt                                    // Beginning Options Column
LOCAL cHoldColor:=SetColor()                     // Save Color Setting
LOCAL nHoldCursor:=SetCursor()                   // Save Cursor Size
LOCAL cHiLiteColor                               // The button Hilite color
LOCAL cCursorPos:=SaveCursorPos()                // Save Cursor Position
LOCAL cHoldScreen                                // Hold under the Dialog Box
LOCAL lHoldSetBlink:=SetBlink()                  // Hold SetBlink Setting
LOCAL nOptSelected                               // The Option Selected, default to 1
LOCAL cShadowColor                               // The Shadow color
LOCAL acNewButtons:={}                           // Array holding the buttons without the ~
LOCAL cHotKeys:=""                               // Concatinated string of Hot keys for the buttons
LOCAL anHotKeyPos:={}                            // Array of hotkey positions in the button for display
LOCAL nLastKey:=0                                // The lastkey that was hit
LOCAL nOldButton                                 // The last button that was hilited
LOCAL cHoldUp                                    // Hold the UpArrow CodeBlock
LOCAL cHoldDown                                  // Hold the DownArrow CodeBlock
LOCAL I, II


   // If the message we have is a character string then make it an array.
   MakeArray(aMessage)
   nTotNumLines:=Len(aMessage)

   // Lets make sure we have at least one button
   IF ( ValType(acOptions) == "U" )
      acOptions:={"Okay"}
   ENDIF         
   nOptions:=Len(acOptions)                      
   Asize(anOptCols, nOptions)
   FOR I:=1 to nOptions                                
      IF ( (II:=At("~", acOptions[I])) > 0 )
         cHotKeys += SubStr(acOptions[I], II + 1, 1)
         Aadd(acNewButtons, StrTran(acOptions[I], "~", ""))
         Aadd(anHotKeyPos, II)
      ELSE
         cHotKeys += Left(Ltrim(acOptions[I]), 1)
         Aadd(acNewButtons, acOptions[I])
         Aadd(anHotKeyPos, Len(acOptions[I]) - Len(Ltrim(acOptions[I])) + 1)
      ENDIF
   NEXT I

   // Lets set some defaults 
   DEFAULT cBoxColor       TO D_BOXCOLOR
   DEFAULT cHdrColor       TO D_HDRCOLOR
   DEFAULT cButtonColor    TO D_BUTTONCOLOR
   DEFAULT lShadow         TO D_SHADOWS
   DEFAULT cHoldUp         TO SetKey(K_UP)
   DEFAULT cHoldDown       TO SetKey(K_DOWN)
   DEFAULT cHotKeyColor    TO D_HOTKEYCOLOR
   DEFAULT nDefaultOption  TO D_DEFAULTOPTION
   DEFAULT nOptSelected    TO nDefaultOption
   DEFAULT cHeader         TO ""
   DEFAULT cFooter         TO ""
   DEFAULT lAllowESC       TO D_ALLOWESC
   DEFAULT lCenterMessages TO .F.
   DEFAULT nBoxBR          TO 0
   DEFAULT nBoxType        TO 1

   CursorOff()
   SetBlink(.F.)

   // Lets find out some colors ahead of time
   cSayColor:=cBoxColor
   cShadowColor:=IIF(At("*", GetPartOfString(cSayColor, ",", 1)) > 0, ;
   "N*/", "N/") + SubStr(cSayColor, At("/", cSayColor) + 1)
   DEFAULT cHiLiteColor TO GetPartOfString(cButtonColor, ",", 2)

   // Lets find the longest Text line to print
   Aeval(aMessage, {|aMessage| nWidestLine:=IIF(aMessage == NIL, nWidestLine, ;
   Max(nWidestLine, Len(aMessage)))})

   // Lets find the length of the Options line
   Aeval(acNewButtons, {|acNewButtons| nLenOfOptions += Len(acNewButtons) + NUMSPACES + 2})

   // Lets now find which has the longest line, the text or options
   nWidestLine:=Max(nWidestLine, nLenOfOptions) + 2

   // If we want the messages centered then center them.
   IF ( lCenterMessage )
      AEval(aMessage, {|cArray, I| aMessage[I]:=PadC(cArray, nWidestLine - 2)})
   ENDIF

   // Lets find the window coordinates
   nBoxTR:=IIF(nBoxTR == NIL, Round( (nMaxRow - (nTotNumLines + 5)) / 2, 0), nBoxTR)
   nBoxTR:=IIF(nBoxTR < 1, 1, nBoxTR)
   nBoxTC:=IIF(nBoxTC == NIL, Round( (nMaxCol - nWidestLine) / 2, 0), nBoxTC)
   IF ( nBoxBR < nBoxTR + 5 )
      nNumLines:=IIF(nBoxTR + nTotNumLines > 19, 19 - nBoxTR, nTotNumLines)
      nBoxBR:=nBoxTR + nNumLines + 4
   ELSE
      nNumLines:=nBoxBR - nBoxTR - 4
   ENDIF
   nBoxBC:=nBoxTC + nWidestLine + 1

   // Save the screen and draw the box below it
   cHoldScreen:=GetScreen(nBoxTR, nBoxTC, nBoxBR + IIF(lShadow, 1, 0), ;
   nBoxBC + IIF(lShadow, 1, 0))
   DrawBox(nBoxTR, nBoxTC, nBoxBR, nBoxBC, nBoxType, cHeader, cFooter, ;
   cBoxColor, cHdrColor, lShadow)

   // Draw the Message lines on the screen
   SetColor(cSayColor)
   Aeval(aMessage, ;
   { |aMessage, nIndex| DevPos(nBoxTR + nIndex, nBoxTC + 2), ;
   DevOut(aMessage)}, 1, nNumLines)

   // Keep track of which lines are currently on the screen.
   nFirstDispLine:=1
   nLastDispLine:=nNumLines

   // Find the first button position
   anOptCols[1]:=nBoxTC + Round((((nBoxBC - nBoxTC) - nLenOfOptions) / 2), 0) + 1

   // Find the rest of the button positions
   FOR I:=2 TO nOptions
      anOptCols[I]:=anOptCols[I-1] + Len(acNewButtons[I - 1]) + NUMSPACES + 2
   NEXT I

   // Draw the buttons on the screen
   SetColor(cButtonColor)
   FOR I:=1 TO nOptions
      @ nBoxBR - 2, anOptCols[I] SAY " " + acNewButtons[I] + " "
      // Show hot key in different color
      SetForeGround(cHotKeyColor)
      @ nBoxBR - 2, anOptCols[I] + anHotKeyPos[I] SAY SubStr(acNewButtons[I], anHotKeyPos[I], 1)
      IF ( lShadow )
         SetColor(cShadowColor)
         @ nBoxBR - 2, anOptCols[I] + LEN(acNewButtons[I]) + 2 SAY ""
         @ nBoxBR - 1, anOptCols[I] + 1 SAY REPLICATE("", LEN(acNewButtons[I]) + 2)
         SetColor(cButtonColor)
      ENDIF
   NEXT I
   SetColor(cHiLiteColor)
   @ nBoxBR - 2, anOptCols[nDefaultOption] SAY LEFTCHAR + acNewButtons[nDefaultOption] + RIGHTCHAR
   SetForeGround(cHotKeyColor)
   @ nBoxBR - 2, anOptCols[nDefaultOption] + anHotKeyPos[nDefaultOption] ;
   SAY SubStr(acNewButtons[nDefaultOption], anHotKeyPos[nDefaultOption], 1)

   DO WHILE ( ( nLastKey != K_ESC ) .AND. ( nLastKey != K_ENTER ) .AND. ;
      ( nLastKey != K_SPACE ) )
      nLastKey:=Inkey(0) 
      nOldButton:=nOptSelected
      DO CASE 
         CASE ( nLastKey == K_RIGHT )
            nOptSelected:=IIF(nOptSelected == nOptions, 1, nOptSelected + 1)
         CASE ( nLastKey == K_LEFT )
            nOptSelected:=IIF(nOptSelected == 1, nOptions, nOptSelected - 1)
         CASE ( nLastKey == K_UP )
            ScrollUp(nBoxTR + 1, nBoxTC + 2, nBoxBR - 4, nBoxBC - 1, aMessage)
         CASE ( nLastKey == K_DOWN )
            ScrollDown(nBoxTR + 1, nBoxTC + 2, nBoxBR - 4, nBoxBC - 1, aMessage)
         CASE ( nLastKey == K_ESC )
            IF ( lAllowEsc )
               nOptSelected:=0          
            ELSE
               nLastKey:=0
            ENDIF
         CASE ( Upper(Chr(nLastKey)) $ cHotKeys )
            nOptSelected:=At(Upper(Chr(nLastKey)), cHotKeys)
            nLastKey:=K_ENTER
      ENDCASE

      // If we changed buttons, dehilite the old one and hilite the new one
      IF ( nOldButton != nOptSelected ) .AND. ( nOptSelected != 0 )
         // First we dehilite
         SetColor(cButtonColor)
         @ nBoxBR - 2, anOptCols[nOldButton] SAY " " + acNewButtons[nOldButton] + " "

         // Now lets draw the hotkey
         SetForeGround(cHotKeyColor)
         @ nBoxBR - 2, anOptCols[nOldButton] + anHotKeyPos[nOldButton] ;
         SAY SubStr(acNewButtons[nOldButton], anHotKeyPos[nOldButton], 1)

         // Hilite the new button
         SetColor(cHiLiteColor)
         @ nBoxBR - 2, anOptCols[nOptSelected] SAY LEFTCHAR + ;
         acNewButtons[nOptSelected] + RIGHTCHAR

         // And draw its hotkey
         SetForeGround(cHotKeyColor)
         @ nBoxBR - 2, anOptCols[nOptSelected] + anHotKeyPos[nOptSelected] ;
         SAY SubStr(acNewButtons[nOptSelected], anHotKeyPos[nOptSelected], 1)
      ENDIF
   ENDDO

   // Only press button down if we elected to have a shadow
   IF ( lShadow )
      IF ( nOptSelected > 0 )
         // Simulate pressing the button down
         SetColor(cSayColor)
         @ nBoxBR - 2, anOptCols[nOptSelected] SAY " "
         @ nBoxBR - 1, anOptCols[nOptSelected] + 1 SAY Replicate(" ", LEN(acNewButtons[nOptSelected]) + 2)
         SetColor(cHiLiteColor)
         @ nBoxBR - 2, anOptCols[nOptSelected] + 1 SAY LEFTCHAR + acNewButtons[nOptSelected] + RIGHTCHAR
         SetForeGround(cHotKeyColor)
         @ nBoxBR - 2, anOptCols[nOptSelected] + anHotKeyPos[nOptSelected] + 1;
         SAY SubStr(acNewButtons[nOptSelected], anHotKeyPos[nOptSelected], 1)
         Inkey(.1)

         // Bring the button back up
         SetColor(cHiLiteColor)
         @ nBoxBR - 2, anOptCols[nOptSelected] SAY LEFTCHAR + acNewButtons[nOptSelected] + RIGHTCHAR
         SetColor(cShadowColor)
         @ nBoxBR - 2, anOptCols[nOptSelected] + LEN(acNewButtons[nOptSelected])  + 2 SAY ""
         @ nBoxBR - 1, anOptCols[nOptSelected] + 1 SAY Replicate("", LEN(acNewButtons[nOptSelected]) + 2)
         SetColor(cHiLiteColor)
         SetForeGround(cHotKeyColor)
         @ nBoxBR - 2, anOptCols[nOptSelected] + anHotKeyPos[nOptSelected] ;
         SAY SubStr(acNewButtons[nOptSelected], anHotKeyPos[nOptSelected], 1)
         Inkey(.1)
      ENDIF
   ENDIF

   // Restore everything back the way we received it
   PutScreen(cHoldScreen)
   RestCursorPos(cCursorPos)
   SetCursor(nHoldCursor)   
   SetColor(cHoldColor)
   SetBlink(lHoldSetBlink)
RETURN ( nOptSelected )


// This function is used to set the foreground color.  Used just for 
// setting the color used for the hot key displaying.  We just want to
// change the foreground color and leave the background color alone which
// is the reason for this function.
*  *
STATIC FUNCTION SetForeGround(cColor)
*  *
   Local cHoldColor:=SetColor()
   SetColor(cColor + SubStr(cHoldColor, At("/", cHoldColor)))
RETURN ( NIL )                                               


// This function is used to scroll down the messages in the dialog box.
*  *
STATIC FUNCTION ScrollDown(nTR, nTC, nBR, nBC, aMessage)
*  *
   LOCAL cHoldColor:=SetColor()
   IF ( nLastDispLine < nTotNumLines )
      SetColor(cSayColor)
      Scroll(nTR, nTC, nBR, nBC, 1)
      @ nBR, nTC SAY aMessage[++nLastDispLine]
      ++nFirstDispLine
      SetColor(cHoldColor)                       
   ENDIF
RETURN ( NIL )


// This function is used to scroll up the messages in the dialog box.
*  *
STATIC FUNCTION ScrollUp(nTR, nTC, nBR, nBC, aMessage)
*  *
   LOCAL cHoldColor:=SetColor()
   IF ( nFirstDispLine > 1 )
      SetColor(cSayColor)
      Scroll(nTR, nTC, nBR, nBC, -1)
      @ nTR, nTC SAY aMessage[--nFirstDispLine]
      --nLastDispLine
      SetColor(cHoldColor)                       
   ENDIF
RETURN ( NIL )


// This function will accept a string with delimeters in it and return
// and array with each element being what was separated with the delimeters.
// For example "BG/R,R/G,B,G/W,W+/R" will return a 5 element array as
// { "BG/R", "R/G", "B", "G/W", "W+/R"}
*  *
FUNCTION BreakOut2Array(cString, cDelimeter)
*  *
   LOCAL I, nFrom:=1, nPos:=1, nStrLen:=0
   LOCAL aRetValue:={}

   nStrLen:=Len(cString)
   DO WHILE ( nPos > 0 )
      nPos:=At(cDelimeter, SubStr(cString, nFrom)) - 1      
      IF ( nPos > 0 )
         Aadd(aRetValue, SubStr(cString, nFrom, nPos))
         nFrom:=nFrom + nPos + 1
      ELSE
         Aadd(aRetValue, SubStr(cString, nFrom))
      ENDIF
   ENDDO
RETURN ( aRetValue )


// This function is used to return only a part of a string separated by
// the delimeters.  
// IE:GetPartOfString("The/Lazy/Dog/Is/Running", "/", 3)    //Result "Dog"
*  *
FUNCTION GetPartOfString(cString, cDelimeter, nPart)
*  *
   LOCAL I, nFrom:=1, nPos:=1, nStrLen:=0, nCurrPart:=0
   LOCAL cRetValue:="", cHoldStr

   nStrLen:=Len(cString)
   DO WHILE ( nCurrPart != nPart .AND. nPos > 0 )
      nPos:=At(cDelimeter, SubStr(cString, nFrom)) - 1      
      IF ( nPos > 0 )
         cHoldStr:=SubStr(cString, nFrom, nPos)
         nFrom+=nPos + 1
      ELSE
         cHoldStr:=SubStr(cString, nFrom)
      ENDIF            
      nCurrPart++
   ENDDO
   IF ( nCurrPart == nPart )
      cRetValue:=cHoldStr
   ENDIF
RETURN ( cRetValue )     


// This function will draw a box on the screen with the an optional
// header and/or footer centered, header/footer colors, shadows, etc.
*  *
FUNCTION DrawBox( nTop, nLeft, nBottom, nRight, nBox, cHeader, cFooter,;
                cBoxColor, cHdrColor, lIsShadow)
*  *
#define BOXDEFAULT 11

   LOCAL aBorders  := { "Ŀ ", ; //  1
                        "ͻȺ ", ; //  2
                        "͸Գ ", ; //  3
                        "ķӺ ", ; //  4
                        " ", ; //  5
                        " ", ; //  6
                        " ", ; //  7
                        " ", ; //  8
                        " ", ; //  9
                        " ", ; // 10
                        "ĿԳ ", ; // 11
                        "³Գ ", ; // 12
                        "ķȺ ", ; // 13
                        "         "  ; // 14
                      }

   // Save the colors and cursor position so we can restore it later.
   LOCAL cHoldColor:=SetColor()
   LOCAL xHoldCursPos:=SaveCursorPos()

   DEFAULT nBox      TO BOXDEFAULT
   DEFAULT cHeader   TO ""
   DEFAULT cBoxColor TO SetColor()
   DEFAULT cHdrColor TO SetColor()
   DEFAULT lIsShadow TO .T.
   
   // First lets draw the box
   SetColor(cBoxColor)
   @ nTop, nLeft, nBottom, nRight BOX aBorders[nBox]
   
   // Now if there is a header, print it
   IF ( !Empty(cHeader) )
      SetColor(cHdrColor)
      @ nTop,  Round((nRight - (nRight - nLeft) / 2) - (Len(cHeader)/2) + .5, 0) SAY cHeader
   ENDIF                      

   // If there is a footer, print it
   IF ( !Empty(cFooter) )
      SetColor(cHdrColor)
      @ nBottom,  Round((nRight - (nRight - nLeft) / 2) - (Len(cFooter)/2) + .5, 0) SAY cFooter
   ENDIF                      

   // Check if we want a shadow, if so then draw it
   IF ( lIsShadow )
      Shadow(nTop, nLeft, nBottom, nRight)
   ENDIF

   // Restore the colors and cursor back the way it was when we got in here.
   SetColor(cHoldColor)
   RestCursorPos(xHoldCursPos)
RETURN ( NIL )


// This function will draw a shadow around the box coordinates.
*  *
STATIC FUNCTION Shadow(nTop, nLeft, nBottom, nRight)
*  *
   #define SHADOW_COLOR       Chr(8)
   LOCAL cHoldScreen
   
   // First lets get the area to shadow
   cHoldScreen:=GetScreen(nTop + 1, nRight + 1, nBottom + 1, nRight + 1)

   // Now lets change the colors to a shadow like color
   cHoldScreen[GS_SCREEN]:=Transform(cHoldScreen[GS_SCREEN], ;
   Replicate("X"+SHADOW_COLOR, Len(cHoldScreen[GS_SCREEN]) / 2 ) )

   // Put the shadow on the screen
   PutScreen(cHoldScreen)

   // First lets get the area to shadow
   cHoldScreen:=GetScreen(nBottom + 1, nLeft + 1, nBottom + 1, nRight)

   // Now lets change the colors to a shadow like color
   cHoldScreen[GS_SCREEN]:=Transform(cHoldScreen[GS_SCREEN], ;
   Replicate("X"+SHADOW_COLOR, Len(cHoldScreen[GS_SCREEN]) / 2 ) )

   // Put the shadow on the screen
   PutScreen(cHoldScreen)
RETURN ( NIL )


// This function when passed an array will return the length of the
// Longest element in the string array.
*  *
FUNCTION ALongest(aArray)
*  *
   LOCAL nLongest:=0
   AEval(aArray, {|aArray, nPos| nLongest:=Max(Len(aArray), nLongest) } )
RETURN ( nLongest )



// This function will in essence act like the Clipper Alert function
// except that you also can set the colors for the buttons, not just
// for the screen.
*  *
FUNCTION ALERT(cMessage, acNewButtons, cDefBoxColors, cDefButtColors)
*  *
#define AL_DEF_BOX_COLOR   "W+/R"
#define AL_DEF_BUTT_COLOR  "N/BG,N/BG*"

   LOCAL aFixedString:=BreakOut2Array(cMessage, ";")  // Convert the string to an array
   LOCAL nLongestElement:=ALongest(aFixedString)      // Find the longest element in the array

   DEFAULT cDefBoxColors TO AL_DEF_BOX_COLOR
   DEFAULT cDefButtColors TO AL_DEF_BUTT_COLOR
   // Now go throught the array and center every string element 
   // to the longest elements length and then call the dialogbox
   AEval(aFixedString, {|cArray, I| aFixedString[I]:=PadC(cArray, nLongestElement)})
RETURN ( DialogBox(aFixedString, acNewButtons, , , cDefBoxColors, , cDefButtColors, .T.)  )
